/*
 * dumpbb.c - Dump netbb format
 *
 * Written 2002 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "dwarf.h"
#include "troll.h"
#include "orc.h"


extern void open_elf(const char *name);
extern const void *mmap_file(const char *name,int *size);

int debug = 0;


static void dump_data(FILE *file,const void *data,int length)
{
    const unsigned char *p = data;
    int i,j;

    for (i = 0; i < length; i += 16) {
	fprintf(file,"%4x: ",i);
	for (j = i; j < i+16 && j < length; j++) {
	    if (!(j & 7)) fputc(' ',file);
	    fprintf(file,"%02x ",p[j]);
	}
	while (j < i+16) {
	    if (!(j & 7)) fputc(' ',file);
	    fprintf(file,"   ");
	    j++;
	}
	for (j = i; j < i+16 && j < length; j++) {
	    if (!(j & 7)) fputc(' ',file);
	    fprintf(file,"%c",*p < ' ' || *p > '~' ? '.' : *p);
	    p++;
	}
	fputc('\n',file);
    }
}


/* ----- Argument processing ----------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [-d] elf_file_name blackbox_file\n",name);
    fprintf(stderr,"%20s-d  enable debugging output\n","");
    exit(1);
}


int main(int argc,char **argv)
{
    const struct troll *type;
    const void *data,*end;
    int c,size;

    while ((c = getopt(argc,argv,"d")) != EOF)
	switch (c) {
	    case 'd':
		debug = 1;
		break;
	    default:
		usage(*argv);
	}
    if (argc != optind+2) usage(*argv);
    open_elf(argv[optind]);
    while (1) {
	const void *p;

	p = dwarf_comp_unit();
	if (!p) break;
	read_types(p);
    }
    data = mmap_file(argv[optind+1],&size);
    end = data+size;
    type = troll_lookup(troll_struct,"sock",-1,0);
    if (!type) {
	fprintf(stderr,"struct sock not found\n");
	exit(1);
    }
    output_data(stdout,type,data);
    data += output_sizeof(type);
    type = troll_lookup(troll_struct,"sk_buff",-1,0);
    if (!type) {
	fprintf(stderr,"struct sk_buff not found\n");
	exit(1);
    }
    while (data != end) {
	int len;

	if (data > end) {
	    fprintf(stderr,"dump file size error\n");
	    exit(1);
	}
	output_data(stdout,type,data);
	len = output_lookup_unsigned(type,data,"len");
	data += output_sizeof(type);
	dump_data(stdout,data,len);
	data += len;
    }
    return 0;
}
