/*
 * troll.h - Types Read frOm eLf fiLe
 *
 * Written 2002 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef TROLL_H
#define TROLL_H

enum troll_type {
    troll_uint,
    troll_sint,
    troll_float,
    troll_enum,
    troll_fn,
    troll_array,
    troll_struct,
    troll_union,
    troll_typedef,
    troll_const,
    troll_ptr,
    troll_volatile,
};

struct troll {
    enum troll_type type;
    const char *name; /* may be NULL */
    int size; /* -1 if unknown */
    union {
	struct {
	    const struct troll *type;
	} modifier;
	struct {
	    int elements; /* -1 if unspecified */
	    const struct troll *type;
	    int stride_size; /* -1 if default */
	} array;
	struct {
	    struct enum_member {
		const char *name;
		int value;
		struct enum_member *next;
	    } *members;
	} _enum;
	struct {
	    struct struct_member {
		const char *name;
		int size;
		unsigned long offset;
		const struct troll *type;
		struct struct_member *next;
	    } *members;
	} _struct; /* ... or union */
	struct {
	    const struct troll *type;
	} _typedef;
	struct {
	    const struct troll *type;
	} fn;
    } u;
    unsigned long def;		/* ELF location of definition (hash key) */
				/* (0 means no key) */
    struct troll *next;		/* next hash collision */
};


/*
 * TYPE is required. If NAME is NULL, entries with any name (or no name) match.
 * If NAME is non-NULL, only entries that have a name can match. If size is -1,
 * entries with any sizes (or unknown size) match. If size is not -1, only
 * entries with that size or unknown size match. SKIP_N specifies the number of
 * matching entries to skip.
 */

const struct troll *troll_lookup(enum troll_type type,const char *name,
  int size,int skip_n);
void read_types(const void *unit);

#endif /* TROLL_H */
